###
### Some internal utility functions. NOT exported.
###

### An utility function to extract basic information about the variables
### stored in an "ncdf" object. The result is returned in a data.frame with
### 1 row per NetCDF variable and 4 columns: name, type, ndims, dim1,
### dim1_name, dim2, dim2_name. If ndims < 2, then some columns will contain
### NAs. If ndims > 2, then those extra dimensions are omitted.
getNcdfVarSummary <- function(nc, varid)
{
    ans_name <- sapply(seq_len(nc$nvars), function(i) nc$var[[i]]$name)
    ans_type <- sapply(seq_len(nc$nvars), function(i) nc$var[[i]]$prec)
    ans_ndims <- sapply(seq_len(nc$nvars), function(i) nc$var[[i]]$ndims)
    get_dim_info <- function(info, j)
    {
        sapply(seq_len(nc$nvars),
               function(i)
                   if (ans_ndims[i] >= j)
                       nc$var[[i]]$dim[[j]][[info]]
                   else
                       NA)
    }
    ans_dim1 <- get_dim_info("len", 1)
    ans_dim1_name <- get_dim_info("name", 1)
    ans_dim2 <- get_dim_info("len", 2)
    ans_dim2_name <- get_dim_info("name", 2)
    data.frame(name = ans_name, type = ans_type, ndims = ans_ndims,
               dim1 = ans_dim1, dim1_name = ans_dim1_name,
               dim2 = ans_dim2, dim2_name = ans_dim2_name,
               stringsAsFactors = FALSE)
}

getGWAScols <- function(nc, first, last = first)
{
    if (!is.numeric(first) || length(first) != 1 || is.na(first))
        stop("'first' must be a single integer")
    if (!is.numeric(last) || length(last) != 1 || is.na(last))
        stop("'last' must be a single integer")
    ncInfo <- getNcdfVarSummary(nc, "snpData")
    ## There might be other variables in addition to the "snpData" var.
    ncInfo <- ncInfo[ncInfo$name == "snpData", ]
    nrows <- ncInfo$dim1
    ncols <- ncInfo$dim2
    if (first < 1 || last < first || last > ncols)
        stop("we need to have 1 <= 'first' <= 'last' <= nb col in NetCDF file")
    dat <- get.var.ncdf(nc, varid = "snpData", start= c(1,first),
            count = c(nrows, last - first + 1))
    structure(as.raw(dat), dim = dim(dat))
}   

